package cn.rankin.apiweb.intercepter;

import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.apiweb.vo.DeviceUserVo;
import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.constant.RedisKey;
import cn.rankin.common.utils.service.RedisService;
import cn.rankin.common.utils.util.HttpUtil;
import com.alibaba.fastjson.JSON;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.NamedThreadLocal;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

@Component
public class LoginInterceptor implements HandlerInterceptor {

    private static final Logger logger = LoggerFactory.getLogger(LoginInterceptor.class);

    private static final String LOGIN_TOKEN_FORMAT_KEY = RedisKey.LOGIN_TOKEN_FORMAT_KEY;

    private static final String ERROR_LOGIN_HEADER = "NO_LOGIN_SIGN";

    private NamedThreadLocal<Long> startTimeThreadLocal = new NamedThreadLocal<Long>("StopWatch-StartTime");

    @Autowired
    private RedisService<String, Object> redisService;

    // 忽略options请求，默认为true
    private boolean ignoreOptions = true;

    @Value("${'${request.header.ignore_path}'.split(',')}")
    private List<String> ignorePaths = new ArrayList<>();

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        //判断 OPTIONS 请求
        if (ignoreOptions && HttpMethod.OPTIONS.matches(request.getMethod())) {
            logger.info("OPTIONS 请求 忽略 返回200");
            request.setAttribute("NO_LOGIN_SIGN", "OPTIONS");
            responseOutWithJson(request, response);
            return false;
        }

        //记录开始时间
        startTimeThreadLocal.set(System.currentTimeMillis());//线程绑定变量（该数据只有当前请求的线程可见）

        //登录请求不拦截
        String path = request.getServletPath();
        if (ignorePaths.contains(path)) {
            logger.info("url: {} not intercepted!");
            return true;
        }

        String uid = request.getHeader("uid");
        String eid = request.getHeader("eid");
        String sign = request.getHeader("sign");
        String requestId = request.getHeader("requestId");
        logger.info("request start, requestId={}, path={}, uid={}, eid={}, sign={}", requestId, path, uid, eid, sign);

        if (StringUtils.isEmpty(uid) || StringUtils.isEmpty(sign)) {
            return false;
        }

        //因为缓存了用户id和设备id
        DeviceUserVo du = (DeviceUserVo) redisService.get(String.format(LOGIN_TOKEN_FORMAT_KEY, uid));
        if (null == du) {
            logger.error("check header failed, not exists!");
            request.setAttribute(ERROR_LOGIN_HEADER, "ERROR_TOKEN");
            responseOutWithJson(request, response);
            return false;
        }

        logger.info("token check success: {}", JSON.toJSONString(du));

        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

        logger.info("请求结束 url={}", request.getRequestURI());
        long beginTime = startTimeThreadLocal.get();//得到线程绑定的局部变量（开始时间）
        long consumeTime = System.currentTimeMillis() - beginTime;//消耗的时间
        if (consumeTime > 500) {//此处认为处理时间超过500毫秒的请求为慢请求
            logger.warn(" 请求: {} consume {} millis", request.getRequestURI(), consumeTime);
        }
    }

    protected void responseOutWithJson(HttpServletRequest request, HttpServletResponse response) {
        BaseCode code = ApiWebCode.SERVER_ERROR;
        logger.error("请求参数详情 {}", JSON.toJSONString(request.getParameterMap()));
        if (!StringUtils.isEmpty(request.getAttribute(ERROR_LOGIN_HEADER))) {
            String sign = (String) request.getAttribute(ERROR_LOGIN_HEADER);
            switch (sign) {
                case "ERROR_REQUEST":
                    code = ApiWebCode.UNAUTHORIZED;
                    break;
                case "ERROR_TOKEN":
                    code = ApiWebCode.INVALID_TOKEN;
                    break;
                case "OPTIONS":
                    code = ApiWebCode.OK;
                    break;
                case "NOT_MATCH":
                    code = ApiWebCode.DEVICE_BOUND_ERROR;
            }
        }
        HttpUtil.responseOutWithJson(request, response, code);
    }

    public void setIgnoreOptions(boolean ignoreOptions) {
        this.ignoreOptions = ignoreOptions;
    }
}
